@@ -205,6 +205,13 @@ static int totemudpu_build_sockets (
 	struct totem_ip_address *bindnet_address,
 	struct totem_ip_address *bound_to);
 
+static int totemudpu_create_sending_socket(
+	void *udpu_context,
+	const struct totem_ip_address *member);
+
+int totemudpu_member_list_rebind_ip (
+	void *udpu_context);
+
 static struct totem_ip_address localhost;
 
 static void totemudpu_instance_initialize (struct totemudpu_instance *instance)
@@ -451,7 +458,7 @@ static int encrypt_and_sign_nss (
 	inbuf = copy_from_iovec(iovec, iov_len, &datalen);
 	if (!inbuf) {
 		log_printf(instance->totemudpu_log_level_security, "malloc error copying buffer from iovec\n");
-		return -1;
+		goto out;
 	}
 
 	data = inbuf + sizeof (struct security_header);
@@ -467,6 +474,7 @@ static int encrypt_and_sign_nss (
 		log_printf(instance->totemudpu_log_level_security,
 			"Failure to generate a random number %d\n",
 			PR_GetError());
+		goto out;
 	}
 
 	memcpy(header->salt, nss_iv_data, sizeof(nss_iv_data));
@@ -482,7 +490,7 @@ static int encrypt_and_sign_nss (
 			"Failure to set up PKCS11 param (err %d)\n",
 			PR_GetError());
 		free (inbuf);
-		return (-1);
+		goto out;
 	}
 
 	/*
@@ -502,7 +510,7 @@ static int encrypt_and_sign_nss (
 			instance->totem_config->crypto_crypt_type,
 			PR_GetError(), err);
 		free(inbuf);
-		return -1;
+		goto sec_out;
 	}
 	rv1 = PK11_CipherOp(enc_context, outdata,
 			    &tmp1_outlen, FRAME_SIZE_MAX - sizeof(struct security_header),
@@ -516,7 +524,7 @@ static int encrypt_and_sign_nss (
 //	memcpy(&outdata[*buf_len], nss_iv_data, sizeof(nss_iv_data));
 
 	if (rv1 != SECSuccess || rv2 != SECSuccess)
-		goto out;
+		goto sec_out;
 
 	/* Now do the digest */
 	enc_context = PK11_CreateContextBySymKey(CKM_SHA_1_HMAC,
@@ -527,7 +535,7 @@ static int encrypt_and_sign_nss (
 		err[PR_GetErrorTextLength()] = 0;
 		log_printf(instance->totemudpu_log_level_security, "encrypt: PK11_CreateContext failed (digest) err %d: %s\n",
 			PR_GetError(), err);
-		return -1;
+		goto sec_out;
 	}
 
 
@@ -539,13 +547,17 @@ static int encrypt_and_sign_nss (
 	PK11_DestroyContext(enc_context, PR_TRUE);
 
 	if (rv1 != SECSuccess || rv2 != SECSuccess)
-		goto out;
+		goto sec_out;
 
 
 	*buf_len = *buf_len + sizeof(struct security_header);
 	SECITEM_FreeItem(nss_sec_param, PR_TRUE);
 	return 0;
 
+sec_out:
+	if (nss_sec_param != NULL) {
+		SECITEM_FreeItem(nss_sec_param, PR_TRUE);
+	}
 out:
 	return -1;
 }
@@ -603,8 +615,7 @@ static int authenticate_and_decrypt_nss (
 		err[PR_GetErrorTextLength()] = 0;
 		log_printf(instance->totemudpu_log_level_security, "PK11_CreateContext failed (check digest) err %d: %s\n",
 			PR_GetError(), err);
-		free (inbuf);
-		return -1;
+		goto out;
 	}
 
 	PK11_DigestBegin(enc_context);
@@ -616,12 +627,12 @@ static int authenticate_and_decrypt_nss (
 
 	if (rv1 != SECSuccess || rv2 != SECSuccess) {
 		log_printf(instance->totemudpu_log_level_security, "Digest check failed\n");
-		return -1;
+		goto out;
 	}
 
 	if (memcmp(digest, header->hash_digest, tmp2_outlen) != 0) {
 		log_printf(instance->totemudpu_log_level_error, "Digest does not match\n");
-		return -1;
+		goto out;
 	}
 
 	/*
@@ -643,7 +654,7 @@ static int authenticate_and_decrypt_nss (
 		log_printf(instance->totemudpu_log_level_security,
 			"PK11_CreateContext (decrypt) failed (err %d)\n",
 			PR_GetError());
-		return -1;
+		goto out;
 	}
 
 	rv1 = PK11_CipherOp(enc_context, outdata, &tmp1_outlen,
@@ -668,6 +679,13 @@ static int authenticate_and_decrypt_nss (
 		return -1;
 
 	return 0;
+
+out:
+	if (iov_len > 1 && inbuf != NULL) {
+		free (inbuf);
+	}
+
+	return (-1);
 }
 #endif
 
@@ -1374,6 +1392,12 @@ static int totemudpu_build_sockets (
 
 	/* We only send out of the token socket */
 	totemudpu_traffic_control_set(instance, instance->token_socket);
+
+	/*
+	 * Rebind all members to new ips
+	 */
+	totemudpu_member_list_rebind_ip(instance);
+
 	return res;
 }
 
@@ -1643,32 +1667,26 @@ extern int totemudpu_recv_mcast_empty (
 	return (msg_processed);
 }
 
-int totemudpu_member_add (
+static int totemudpu_create_sending_socket(
 	void *udpu_context,
 	const struct totem_ip_address *member)
 {
 	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
-
-	struct totemudpu_member *new_member;
+	int fd;
 	int res;
 	unsigned int sendbuf_size;
 	unsigned int optlen = sizeof (sendbuf_size);
+	struct sockaddr_storage sockaddr;
+	int addrlen;
 
-	new_member = malloc (sizeof (struct totemudpu_member));
-	if (new_member == NULL) {
-		return (-1);
-	}
-	list_init (&new_member->list);
-	list_add_tail (&new_member->list, &instance->member_list);
-	memcpy (&new_member->member, member, sizeof (struct totem_ip_address));
-	new_member->fd = socket (member->family, SOCK_DGRAM, 0);
-	if (new_member->fd == -1) {
+	fd = socket (member->family, SOCK_DGRAM, 0);
+	if (fd == -1) {
 		LOGSYS_PERROR (errno, instance->totemudpu_log_level_warning,
 			"Could not create socket for new member");
 		return (-1);
 	}
-	totemip_nosigpipe (new_member->fd);
-	res = fcntl (new_member->fd, F_SETFL, O_NONBLOCK);
+	totemip_nosigpipe (fd);
+	res = fcntl (fd, F_SETFL, O_NONBLOCK);
 	if (res == -1) {
 		LOGSYS_PERROR (errno, instance->totemudpu_log_level_warning,
 			"Could not set non-blocking operation on token socket");
@@ -1680,12 +1698,47 @@ int totemudpu_member_add (
  	 * should be large
  	 */
 	sendbuf_size = MCAST_SOCKET_BUFFER_SIZE;
-	res = setsockopt (new_member->fd, SOL_SOCKET, SO_SNDBUF,
+	res = setsockopt (fd, SOL_SOCKET, SO_SNDBUF,
 		&sendbuf_size, optlen);
 	if (res == -1) {
 		LOGSYS_PERROR (errno, instance->totemudpu_log_level_notice,
 			"Could not set sendbuf size");
 	}
+
+	/*
+	 * Bind to sending interface
+	 */
+	totemip_totemip_to_sockaddr_convert(&instance->my_id, 0, &sockaddr, &addrlen);
+	res = bind (fd, (struct sockaddr *)&sockaddr, addrlen);
+	if (res == -1) {
+		LOGSYS_PERROR (errno, instance->totemudpu_log_level_warning,
+			"bind token socket failed");
+		return (-1);
+	}
+
+	return (fd);
+
+}
+
+int totemudpu_member_add (
+	void *udpu_context,
+	const struct totem_ip_address *member)
+{
+	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
+
+	struct totemudpu_member *new_member;
+
+	new_member = malloc (sizeof (struct totemudpu_member));
+	if (new_member == NULL) {
+		return (-1);
+	}
+	log_printf (LOGSYS_LEVEL_NOTICE, "adding new UDPU member {%s}",
+		totemip_print(member));
+	list_init (&new_member->list);
+	list_add_tail (&new_member->list, &instance->member_list);
+	memcpy (&new_member->member, member, sizeof (struct totem_ip_address));
+	new_member->fd = totemudpu_create_sending_socket(udpu_context, member);
+
 	return (0);
 }
 
@@ -1698,3 +1751,29 @@ int totemudpu_member_remove (
 	instance = NULL;
 	return (0);
 }
+
+int totemudpu_member_list_rebind_ip (
+	void *udpu_context)
+{
+	struct list_head *list;
+	struct totemudpu_member *member;
+
+	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
+
+	for (list = instance->member_list.next;
+		list != &instance->member_list;
+		list = list->next) {
+
+		member = list_entry (list,
+			struct totemudpu_member,
+			list);
+
+		if (member->fd > 0) {
+			close (member->fd);
+		}
+
+		member->fd = totemudpu_create_sending_socket(udpu_context, &member->member);
+	}
+
+	return (0);
+}
