@@ -1,13 +1,13 @@
 /*
  * Copyright (c) 2004 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006-2007, 2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -46,20 +46,21 @@
 #include <corosync/evs.h>
 #include "../exec/crypto.h"
 
-char *delivery_string;
+const char *delivery_string;
 struct my_msg {
 	unsigned int msg_size;
 	unsigned char sha1[20];
 	unsigned char buffer[0];
 };
 
-int deliveries = 0;
-void evs_deliver_fn (
+static int deliveries = 0;
+static void evs_deliver_fn (
+	hdb_handle_t handle,
 	unsigned int nodeid,
-	void *m,
-	int msg_len)
+	const void *m,
+	size_t msg_len)
 {
-	struct my_msg *msg2 = (struct my_msg *)m;
+	const struct my_msg *msg2 = m;
 	unsigned char sha1_compare[20];
 	hash_state sha1_hash;
 	unsigned int i;
@@ -80,10 +81,12 @@ printf ("\n");
 	deliveries++;
 }
 
-void evs_confchg_fn (
-	unsigned int *member_list, int member_list_entries,
-	unsigned int *left_list, int left_list_entries,
-	unsigned int *joined_list, int joined_list_entries)
+static void evs_confchg_fn (
+	hdb_handle_t handle,
+	const unsigned int *member_list, size_t member_list_entries,
+	const unsigned int *left_list, size_t left_list_entries,
+	const unsigned int *joined_list, size_t joined_list_entries,
+	const struct evs_ring_id *ring_id)
 {
 	int i;
 
@@ -103,7 +106,7 @@ void evs_confchg_fn (
 	}
 }
 
-evs_callbacks_t callbacks = {
+static evs_callbacks_t callbacks = {
 	evs_deliver_fn,
 	evs_confchg_fn
 };
@@ -114,9 +117,7 @@ struct evs_group groups[3] = {
 	{ "key3" }
 };
 
-struct my_msg msg;
-
-unsigned char buffer[200000];
+static unsigned char buffer[200000];
 int main (void)
 {
 	evs_handle_t handle;
@@ -125,7 +126,7 @@ int main (void)
 	int fd;
 	unsigned int member_list[32];
 	unsigned int local_nodeid;
-	unsigned int member_list_entries = 32;
+	size_t member_list_entries = 32;
 	struct my_msg msg;
 	hash_state sha1_hash;
 	struct iovec iov[2];
@@ -135,11 +136,11 @@ int main (void)
 		printf ("Couldn't initialize EVS service %d\n", result);
 		exit (0);
 	}
-	
+
 	result = evs_membership_get (handle, &local_nodeid,
 		member_list, &member_list_entries);
-	printf ("Current membership from evs_membership_get entries %d\n",
-		member_list_entries);
+	printf ("Current membership from evs_membership_get entries %lu\n",
+		(unsigned long int) member_list_entries);
 	for (i = 0; i < member_list_entries; i++) {
 		printf ("member [%d] is %x\n", i, member_list[i]);
 	}
@@ -152,9 +153,9 @@ int main (void)
 	printf ("Leave result %d\n", result);
 	delivery_string = "evs_mcast_joined";
 
-	iov[0].iov_base = &msg;
+	iov[0].iov_base = (void *)&msg;
 	iov[0].iov_len = sizeof (struct my_msg);
-	iov[1].iov_base = buffer;
+	iov[1].iov_base = (void *)buffer;
 
 	/*
 	 * Demonstrate evs_mcast_joined
@@ -180,7 +181,7 @@ try_again_one:
 	}
 
 	evs_fd_get (handle, &fd);
-	
+
 	evs_finalize (handle);
 
 	return (0);
