@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2008 Allied Telesis Labs NZ
+ *                 (c) 2012 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -49,7 +50,7 @@
 
 #define SEPERATOR '.'
 #define SEPERATOR_STR "."
-#define OBJ_NAME_SIZE 512
+#define OBJ_NAME_SIZE 4096
 
 
 typedef enum {
@@ -93,7 +94,6 @@ static void tail_object_deleted(confdb_handle_t handle,
 
 static void create_object(confdb_handle_t handle, char * name_pt);
 static void create_object_key(confdb_handle_t handle, char * name_pt);
-static void destroy_object_key(confdb_handle_t handle, char * name_pt);
 static void write_key(confdb_handle_t handle, char * path_pt);
 static void get_parent_name(const char * name_pt, char * parent_name);
 
@@ -104,8 +104,29 @@ static confdb_callbacks_t callbacks = {
 };
 
 static int debug = 0;
+static int show_binary = 0;
 static int action;
 
+static void print_binary_key (char *value, size_t value_len)
+{
+	size_t i;
+	char c;
+
+	for (i = 0; i < value_len; i++) {
+		c = value[i];
+		if (c >= ' ' && c < 0x7f && c != '\\') {
+			fputc (c, stdout);
+		} else {
+			if (c == '\\') {
+				printf ("\\\\");
+			} else {
+				printf ("\\x%02X", c);
+			}
+		}
+	}
+	printf ("\n");
+}
+
 static void print_key (char *key_name, void *value, size_t value_len, confdb_value_types_t type)
 {
 	switch (type) {
@@ -146,7 +167,12 @@ static void print_key (char *key_name, void *value, size_t value_len, confdb_val
 			break;
 		default:
 		case CONFDB_VALUETYPE_ANY:
-			printf ("%s=**binary**(%d)\n", key_name, type);
+			if (!show_binary) {
+				printf ("%s=**binary**(%d)\n", key_name, type);
+			} else {
+				printf ("%s=", key_name);
+				print_binary_key ((char *)value, value_len);
+			}
 			break;
 	}
 }
@@ -158,7 +184,8 @@ static void print_config_tree(confdb_handle_t handle, hdb_handle_t parent_object
 	char object_name[OBJ_NAME_SIZE];
 	size_t object_name_len;
 	char key_name[OBJ_NAME_SIZE];
-	char key_value[OBJ_NAME_SIZE];
+	char output_string[OBJ_NAME_SIZE];
+	char *key_value=NULL;/* Dynamically allocated value */
 	size_t key_value_len;
 	cs_error_t res;
 	int children_printed;
@@ -172,10 +199,10 @@ static void print_config_tree(confdb_handle_t handle, hdb_handle_t parent_object
 	}
 	children_printed = 0;
 
-	while ( (res = confdb_key_iter_typed(handle,
+	while ( (res = confdb_key_iter_typed2(handle,
 								   parent_object_handle,
 								   key_name,
-								   key_value,
+					                           (void**)&key_value,
 								   &key_value_len,
 								   &type)) == CS_OK) {
 		key_value[key_value_len] = '\0';
@@ -183,6 +210,8 @@ static void print_config_tree(confdb_handle_t handle, hdb_handle_t parent_object
 			printf("%s%c", parent_name, SEPERATOR);
 
 		print_key(key_name, key_value, key_value_len, type);
+		free(key_value);
+		key_value = NULL;
 
 		children_printed++;
 	}
@@ -202,12 +231,12 @@ static void print_config_tree(confdb_handle_t handle, hdb_handle_t parent_object
 
 		object_name[object_name_len] = '\0';
 		if (parent_name != NULL) {
-			snprintf(key_value, OBJ_NAME_SIZE, "%s%c%s", parent_name, SEPERATOR, object_name);
+			snprintf(output_string, OBJ_NAME_SIZE, "%s%c%s", parent_name, SEPERATOR, object_name);
 		} else {
 			if ((action == ACTION_PRINT_DEFAULT) && strcmp(object_name, "internal_configuration") == 0) continue;
-			snprintf(key_value, OBJ_NAME_SIZE, "%s", object_name);
+			snprintf(output_string, OBJ_NAME_SIZE, "%s", object_name);
 		}
-		print_config_tree(handle, object_handle, key_value);
+		print_config_tree(handle, object_handle, output_string);
 		children_printed++;
 	}
 	if (children_printed == 0 && parent_name != NULL) {
@@ -215,6 +244,18 @@ static void print_config_tree(confdb_handle_t handle, hdb_handle_t parent_object
 	}
 }
 
+static cs_error_t reload_db(confdb_handle_t handle)
+{
+	char error[1024];
+	cs_error_t ret;
+
+	ret = confdb_reload(handle, 0, error, sizeof(error) - 1);
+	if (ret != CONFDB_OK) {
+		fprintf (stderr, "Error reloading DB %d\n", ret);
+	}
+	return ret;
+}
+
 static int read_in_config_file (char * filename)
 {
 	confdb_handle_t handle;
@@ -285,7 +326,7 @@ static int read_in_config_file (char * filename)
 		/* write the attribute */
 		write_key (handle, line);
 	}
-
+	reload_db(handle);
 	confdb_finalize (handle);
 	fclose (fh);
 	return 0;
@@ -313,13 +354,13 @@ static int print_all(void)
 static int print_help(void)
 {
 	printf("\n");
-	printf ("usage:  corosync-objctl object%ckey ...                    Print an object\n", SEPERATOR);
+	printf ("usage:  corosync-objctl [-b] object%ckey ...               Print an object\n", SEPERATOR);
 	printf ("        corosync-objctl -c object%cchild_obj ...           Create Object\n", SEPERATOR);
 	printf ("        corosync-objctl -d object%cchild_obj ...           Delete object\n", SEPERATOR);
 	printf ("        corosync-objctl -w object%cchild_obj.key=value ... Create a key\n", SEPERATOR);
 	printf ("        corosync-objctl -n object%cchild_obj.key=value ... Create a new object with the key\n", SEPERATOR);
 	printf ("        corosync-objctl -t object%cchild_obj ...           Track changes\n", SEPERATOR);
-	printf ("        corosync-objctl -a                                Print all objects\n");
+	printf ("        corosync-objctl [-b] -a                           Print all objects\n");
 	printf ("        corosync-objctl -p <filename> Load in config from the specified file.\n");
 	printf("\n");
 	return 0;
@@ -407,7 +448,8 @@ static cs_error_t find_object (confdb_handle_t handle,
 	char tmp_name[OBJ_NAME_SIZE];
 	cs_error_t res = CS_OK;
 
-	strncpy (tmp_name, name_pt, OBJ_NAME_SIZE);
+	strncpy (tmp_name, name_pt, sizeof (tmp_name));
+	tmp_name[sizeof (tmp_name) - 1] = '\0';
 	obj_name_pt = strtok_r(tmp_name, SEPERATOR_STR, &save_pt);
 
 	while (obj_name_pt != NULL) {
@@ -451,6 +493,7 @@ static void write_key(confdb_handle_t handle, char * path_pt)
 	char key_name[OBJ_NAME_SIZE];
 	char key_value[OBJ_NAME_SIZE];
 	char old_key_value[OBJ_NAME_SIZE];
+	char *old_key_value_ptr = old_key_value;
 	size_t old_key_value_len;
 	cs_error_t res;
 	confdb_value_types_t type;
@@ -475,12 +518,16 @@ static void write_key(confdb_handle_t handle, char * path_pt)
 	}
 
 	/* get the current key */
-	res = confdb_key_get_typed (handle,
+	res = confdb_key_get_typed2 (handle,
 						  obj_handle,
 						  key_name,
-						  old_key_value,
+				                  (void**)&old_key_value_ptr,
 						  &old_key_value_len, &type);
 
+	if (debug == 1 && res==CS_OK)
+		printf ("%d: key:\"%s\", old value:\"%s\"\n",
+				__LINE__, key_name, old_key_value);
+
 	if (res == CS_OK) {
 		/* replace the current value */
 		res = confdb_key_replace (handle,
@@ -490,7 +537,7 @@ static void write_key(confdb_handle_t handle, char * path_pt)
 								  old_key_value,
 								  old_key_value_len,
 								  key_value,
-								  strlen(key_value));
+								  strlen(key_value)+1);
 
 		if (res != CS_OK)
 			fprintf(stderr, "Failed to replace the key %s=%s. Error %d\n", key_name, key_value, res);
@@ -500,7 +547,7 @@ static void write_key(confdb_handle_t handle, char * path_pt)
 								 obj_handle,
 								 key_name,
 								 key_value,
-								 strlen(key_value),
+								 strlen(key_value)+1,
 								 CONFDB_VALUETYPE_STRING);
 		if (res != CS_OK)
 			fprintf(stderr, "Failed to create the key %s=%s. Error %d\n", key_name, key_value, res);
@@ -517,7 +564,8 @@ static void create_object(confdb_handle_t handle, char * name_pt)
 	char tmp_name[OBJ_NAME_SIZE];
 	cs_error_t res;
 
-	strncpy (tmp_name, name_pt, OBJ_NAME_SIZE);
+	strncpy (tmp_name, name_pt, sizeof (tmp_name));
+	tmp_name[sizeof (tmp_name) - 1] = '\0';
 	obj_name_pt = strtok_r(tmp_name, SEPERATOR_STR, &save_pt);
 
 	while (obj_name_pt != NULL) {
@@ -570,7 +618,8 @@ static void create_object_key(confdb_handle_t handle, char *name_pt)
 	get_parent_name(name_pt, parent_name);
 	get_key(name_pt, key_name, key_value);
 
-	strncpy (tmp_name, parent_name, OBJ_NAME_SIZE);
+	strncpy (tmp_name, parent_name, sizeof (tmp_name));
+	tmp_name[sizeof (tmp_name) - 1] = '\0';
 	obj_name_pt = strtok_r(tmp_name, SEPERATOR_STR, &save_pt);
 
 	/*
@@ -617,7 +666,7 @@ static void create_object_key(confdb_handle_t handle, char *name_pt)
 		obj_handle,
 		key_name,
 		key_value,
-		strlen(key_value),
+		strlen(key_value)+1,
 		CONFDB_VALUETYPE_STRING);
 	if (res != CS_OK) {
 		fprintf(stderr,
@@ -800,7 +849,7 @@ int main (int argc, char *argv[]) {
 	action = ACTION_READ;
 
 	for (;;){
-		c = getopt (argc,argv,"hawncvdtp:");
+		c = getopt (argc,argv,"habwncvdtp:");
 		if (c==-1) {
 			break;
 		}
@@ -814,6 +863,9 @@ int main (int argc, char *argv[]) {
 			case 'a':
 				action = ACTION_PRINT_ALL;
 				break;
+			case 'b':
+				show_binary++;
+				break;
 			case 'p':
 				return read_in_config_file (optarg);
 				break;
@@ -879,6 +931,8 @@ int main (int argc, char *argv[]) {
 	if (action == ACTION_TRACK) {
 		listen_for_object_changes(handle);
 		stop_tracking(handle);
+	} else {
+		reload_db(handle);
 	}
 
 	result = confdb_finalize (handle);
